#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_portal.interfaces import IPortalTemplateContainer, IPortalTemplate, IPortalContext, \
    MANAGE_TEMPLATE_PERMISSION, LOCAL_TEMPLATE_NAME
from pyams_skin.interfaces import IPageHeader, IContentTitle
from pyams_skin.interfaces.container import ITableElementName, ITableElementEditor
from pyams_skin.interfaces.viewlet import IWidgetTitleViewletManager, IBreadcrumbItem
from pyams_skin.layer import IPyAMSLayer
from pyams_zmi.layer import IAdminLayer
from z3c.form.interfaces import IDataExtractedEvent
from zope.component.interfaces import ISite

# import packages
from pyams_form.form import AJAXAddForm, ajax_config
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.template import PortalTemplate
from pyams_portal.zmi.container import PortalTemplateContainerTable
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.table import DefaultElementEditorAdapter
from pyams_skin.viewlet.breadcrumb import BreadcrumbAdminLayerItem
from pyams_skin.viewlet.toolbar import ToolbarAction
from pyams_utils.adapter import adapter_config, ContextRequestAdapter
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_utils.unicode import translate_string
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm
from pyramid.events import subscriber
from z3c.form import field
from zope.interface import Interface, Invalid

from pyams_portal import _


@adapter_config(context=(IPortalTemplate, IAdminLayer), provides=IBreadcrumbItem)
class PortalTemplateBreadcrumbAdapter(BreadcrumbAdminLayerItem):
    """Portal template breadcrumb adapter"""

    @property
    def label(self):
        name = self.context.name
        if name == LOCAL_TEMPLATE_NAME:
            return _("Presentation template")
        else:
            return name

    @property
    def url(self):
        name = self.context.name
        if name == LOCAL_TEMPLATE_NAME:
            parent = get_parent(self.context, IPortalContext)
            return absolute_url(parent, self.request, 'admin#template-properties.html')
        else:
            return super(PortalTemplateBreadcrumbAdapter, self).url


@adapter_config(context=(IPortalTemplate, IPyAMSLayer), provides=ITableElementName)
class PortalTemplateTableElementNameAdapter(ContextRequestAdapter):
    """Portal template table element name adapter"""

    @property
    def name(self):
        container = get_parent(self.context, IPortalTemplateContainer)
        if container is not None:
            return self.context.name
        context = get_parent(self.context, IPortalContext)
        if context is not None:
            adapter = self.request.registry.queryMultiAdapter((context, self.request), ITableElementName)
            if adapter is not None:
                return self.request.localizer.translate(_("{0} (local template)")).format(adapter.name)
        return '--'


@adapter_config(context=(IPortalTemplate, IAdminLayer, Interface), provides=ITableElementEditor)
class PortalTemplateTableElementEditorAdapter(DefaultElementEditorAdapter):
    """Portal template table element editor adapter"""

    view_name = 'admin#properties.html'
    modal_target = False


@adapter_config(context=(IPortalTemplate, IPyAMSLayer, Interface), provides=IPageHeader)
class PortalTemplateHeaderAdapter(DefaultPageHeaderAdapter):
    """Portal template header adapter"""

    icon_class = 'fa fa-fw fa-columns'

    @property
    def title(self):
        translate = self.request.localizer.translate
        # check for templates container
        container = get_parent(self.context, IPortalTemplateContainer)
        if container is not None:
            return translate(_("« {0} »  portal template")).format(self.context.name)
        # check for portal context
        context = get_parent(self.context, IPortalContext)
        if context is not None:
            adapter = self.request.registry.queryMultiAdapter((context, self.request, self.view), IContentTitle)
            if adapter is None:
                adapter = IContentTitle(context, None)
            if adapter is not None:
                return adapter.title


#
# Template views
#

@viewlet_config(name='add-portal-template.action', context=ISite, layer=IAdminLayer,
                view=PortalTemplateContainerTable, manager=IWidgetTitleViewletManager,
                permission=MANAGE_TEMPLATE_PERMISSION, weight=1)
class PortalTemplateAddAction(ToolbarAction):
    """Portal template add action"""

    label = _("Add template")
    url = 'add-portal-template.html'
    modal_target = True


@pagelet_config(name='add-portal-template.html', context=ISite, layer=IPyAMSLayer,
                permission=MANAGE_TEMPLATE_PERMISSION)
@ajax_config(name='add-portal-template.json', context=ISite, layer=IPyAMSLayer, base=AJAXAddForm)
class PortalTemplateAddForm(AdminDialogAddForm):
    """Portal template add form"""

    title = _("Portal templates")
    legend = _("Add shared template")
    icon_css_class = 'fa fa-fw fa-columns'

    fields = field.Fields(IPortalTemplate)
    edit_permission = MANAGE_TEMPLATE_PERMISSION

    def create(self, data):
        return PortalTemplate()

    def add(self, template):
        context = query_utility(IPortalTemplateContainer)
        context[translate_string(template.name, spaces='-')] = template

    def nextURL(self):
        return absolute_url(self.context, self.request, 'portal-templates.html')


@subscriber(IDataExtractedEvent, form_selector=PortalTemplateAddForm)
def handle_new_template_data_extraction(event):
    """Handle new template form data extraction"""
    container = query_utility(IPortalTemplateContainer)
    name = event.data.get('name')
    if name in container:
        event.form.widgets.errors += (Invalid(_("Specified name is already used!")),)
